"use strict";
/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.initModel = exports.getReference = exports.getSdfTrajectory = void 0;
var tslib_1 = require("tslib");
var parser_1 = require("../../mol-io/reader/sdf/parser");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var mol_1 = require("../../mol-model-formats/structure/mol");
var structure_1 = require("../../mol-model/structure");
var trajectory_1 = require("../../mol-model/structure/trajectory");
var ball_and_stick_1 = require("../../mol-repr/structure/representation/ball-and-stick");
var mol_task_1 = require("../../mol-task");
var color_1 = require("../../mol-theme/color");
var size_1 = require("../../mol-theme/size");
function getSdfTrajectory(url) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var data, parsed, _a, models, _i, _b, molFile, traj, i, _c, _d;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0: return [4 /*yield*/, fetch(url)];
                case 1:
                    data = _e.sent();
                    _a = parser_1.parseSdf;
                    return [4 /*yield*/, data.text()];
                case 2: return [4 /*yield*/, _a.apply(void 0, [_e.sent()]).run()];
                case 3:
                    parsed = _e.sent();
                    if (parsed.isError)
                        throw parsed;
                    models = [];
                    _i = 0, _b = parsed.result.compounds;
                    _e.label = 4;
                case 4:
                    if (!(_i < _b.length)) return [3 /*break*/, 10];
                    molFile = _b[_i].molFile;
                    return [4 /*yield*/, mol_1.trajectoryFromMol(molFile).run()];
                case 5:
                    traj = _e.sent();
                    i = 0;
                    _e.label = 6;
                case 6:
                    if (!(i < traj.frameCount)) return [3 /*break*/, 9];
                    _d = (_c = models).push;
                    return [4 /*yield*/, mol_task_1.Task.resolveInContext(traj.getFrameAtIndex(i))];
                case 7:
                    _d.apply(_c, [_e.sent()]);
                    _e.label = 8;
                case 8:
                    i++;
                    return [3 /*break*/, 6];
                case 9:
                    _i++;
                    return [3 /*break*/, 4];
                case 10: return [2 /*return*/, new trajectory_1.ArrayTrajectory(models)];
            }
        });
    });
}
exports.getSdfTrajectory = getSdfTrajectory;
var colorThemeRegistry = color_1.ColorTheme.createRegistry();
var sizeThemeRegistry = size_1.SizeTheme.createRegistry();
function getReference(traj, i) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var model, structure;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, mol_task_1.Task.resolveInContext(traj.getFrameAtIndex(i))];
                case 1:
                    model = _a.sent();
                    structure = structure_1.Structure.ofModel(model);
                    return [2 /*return*/, structure.boundary.sphere];
            }
        });
    });
}
exports.getReference = getReference;
function initModel(i, canvas3d, traj, ref) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var reprCtx, model, _structure, translation, transform, structure, labelElem, ballAndStickRepr;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    reprCtx = { webgl: canvas3d.webgl, colorThemeRegistry: colorThemeRegistry, sizeThemeRegistry: sizeThemeRegistry };
                    return [4 /*yield*/, mol_task_1.Task.resolveInContext(traj.getFrameAtIndex(i))];
                case 1:
                    model = _a.sent();
                    _structure = structure_1.Structure.ofModel(model);
                    translation = linear_algebra_1.Vec3.sub(linear_algebra_1.Vec3(), ref, _structure.boundary.sphere.center);
                    transform = linear_algebra_1.Mat4.fromTranslation(linear_algebra_1.Mat4(), translation);
                    structure = structure_1.Structure.transform(_structure, transform);
                    labelElem = document.getElementById("label" + i);
                    labelElem.innerText = model.entities.data.pdbx_description.toArray()[0].join(', ');
                    ballAndStickRepr = ball_and_stick_1.BallAndStickRepresentationProvider.factory(reprCtx, ball_and_stick_1.BallAndStickRepresentationProvider.getParams);
                    ballAndStickRepr.setTheme({
                        color: reprCtx.colorThemeRegistry.create('element-symbol', { structure: structure }, { carbonColor: 'elementSymbol' }),
                        size: reprCtx.sizeThemeRegistry.create('physical', { structure: structure }, { value: 1 })
                    });
                    return [4 /*yield*/, ballAndStickRepr.createOrUpdate(tslib_1.__assign(tslib_1.__assign({}, ball_and_stick_1.BallAndStickRepresentationProvider.defaultValues), { quality: 'medium' }), structure).run()];
                case 2:
                    _a.sent();
                    canvas3d.add(ballAndStickRepr);
                    return [2 /*return*/];
            }
        });
    });
}
exports.initModel = initModel;
//# sourceMappingURL=model.js.map