/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Canvas3D, PartialCanvas3DProps } from '../../mol-canvas3d/canvas3d';
import { PickingId } from '../../mol-geo/geometry/picking';
import { WebGLContext } from '../../mol-gl/webgl/context';
import { Vec3 } from '../../mol-math/linear-algebra';
import { Representation } from '../../mol-repr/representation';
import { InputObserver } from '../../mol-util/input/input-observer';
import { MarkerAction } from '../../mol-util/marker-action';
export interface MultiCanvas3DBuilder {
    parent: Element;
    elementCount: number;
    getElement(i: number): Element;
    props: (i: number) => PartialCanvas3DProps;
    init: (i: number, canvas3d: Canvas3D) => void;
}
export declare class MultiCanvas3D {
    private builder;
    private ev;
    readonly interaction: {
        hover: import("rxjs").Subject<Canvas3D.HoverEvent>;
        drag: import("rxjs").Subject<Canvas3D.DragEvent>;
        click: import("rxjs").Subject<Canvas3D.ClickEvent>;
    };
    readonly input: InputObserver;
    readonly webgl: WebGLContext;
    readonly grid: (Canvas3D | undefined)[];
    private readonly container;
    private readonly canvas;
    private readonly passes;
    private currentCell;
    constructor(builder: MultiCanvas3DBuilder);
    private updateCurrentCell;
    readonly refOrientation: Orientation;
    private cameraSyncState;
    get cameraSync(): boolean;
    set cameraSync(state: boolean);
    private resetCamera;
    private syncOrientation;
    private readonly contextLost;
    private handleWebglContextLost;
    private handlewWebglContextRestored;
    draw(): void;
    handleResize(): void;
    animate(): void;
    pause(): void;
    identify(x: number, y: number): import("../../mol-canvas3d/passes/pick").PickData | undefined;
    mark(loci: Representation.Loci, action: MarkerAction): void;
    getLoci(pickingId: PickingId): Representation.Loci<import("../../mol-model/loci").Loci>;
    setProps(props: PartialCanvas3DProps): void;
    dispose(options?: Partial<{
        doNotForceWebGLContextLoss: boolean;
    }>): void;
}
declare type Orientation = {
    position: Vec3;
    up: Vec3;
    target: Vec3;
};
export {};
