/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __awaiter, __generator } from "tslib";
import './index.html';
import { Camera } from '../../mol-canvas3d/camera';
import { Vec3 } from '../../mol-math/linear-algebra';
import { Loci } from '../../mol-model/loci';
import { lociLabel } from '../../mol-theme/label';
import { getSdfTrajectory, getReference, initModel } from './model';
import { MultiCanvas3D } from './multi-canvas3d';
var example = { count: 1000, url: '../../../examples/nci_download.sdf' };
var box = document.getElementById('box');
var info = document.getElementById('info');
var sync = document.getElementById('sync');
var container = document.getElementById('container');
var cards = [];
for (var i = 0; i < example.count; ++i) {
    var card = document.createElement('div');
    container.appendChild(card);
    card.className = 'card';
    var viewer = document.createElement('div');
    card.appendChild(viewer);
    viewer.className = 'viewer';
    var label = document.createElement('div');
    card.appendChild(label);
    label.className = 'label';
    label.textContent = "Molecule " + (i + 1);
    label.id = "label" + i;
    cards.push(viewer);
}
var builder = {
    parent: box,
    elementCount: cards.length,
    getElement: function (i) { return cards[i]; },
    props: function (_) { return ({
        camera: { helper: { axes: { name: 'off', params: {} } }, manualReset: true },
        trackball: { spin: spinState },
        renderer: { style: { name: 'glossy', params: {} } },
        postprocessing: {
        // occlusion: { name: 'off', params: {} },
        // antialiasing: { name: 'off', params: {} },
        },
    }); },
    init: function (i, canvas3d) {
        initModel(i, canvas3d, traj, ref.center);
    }
};
var canvas3d = new MultiCanvas3D(builder);
var spinState = false;
canvas3d.input.key.subscribe(function (_a) {
    var key = _a.key;
    if (key !== 's')
        return;
    spinState = !spinState;
    canvas3d.setProps({ trackball: { spin: spinState } });
});
canvas3d.interaction.hover.subscribe(function (e) {
    info.innerHTML = Loci.isEmpty(e.current.loci) ? '' : lociLabel(e.current.loci);
});
sync.onclick = function () {
    canvas3d.cameraSync = !canvas3d.cameraSync;
    sync.innerText = "Sync Orientation: " + (canvas3d.cameraSync ? 'On' : 'Off');
};
sync.innerText = "Sync Orientation: " + (canvas3d.cameraSync ? 'On' : 'Off');
var traj;
var ref;
function init() {
    return __awaiter(this, void 0, void 0, function () {
        var _a, position, target, fov, _b, width, height, d, deltaDirection;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0: return [4 /*yield*/, getSdfTrajectory(example.url)];
                case 1:
                    traj = _c.sent();
                    return [4 /*yield*/, getReference(traj, 0)];
                case 2:
                    ref = _c.sent();
                    _a = canvas3d.refOrientation, position = _a.position, target = _a.target;
                    Vec3.copy(target, ref.center);
                    fov = Math.PI / 4;
                    _b = builder.getElement(0).getBoundingClientRect(), width = _b.width, height = _b.height;
                    d = Camera.targetDistance(ref.radius + 0.5, fov, width, height);
                    deltaDirection = Vec3.create(0, 0, 1);
                    Vec3.setMagnitude(deltaDirection, deltaDirection, d);
                    Vec3.add(position, target, deltaDirection);
                    canvas3d.draw();
                    return [2 /*return*/];
            }
        });
    });
}
init();
//# sourceMappingURL=index.js.map