/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __assign, __awaiter, __generator } from "tslib";
import { parseSdf } from '../../mol-io/reader/sdf/parser';
import { Mat4, Vec3 } from '../../mol-math/linear-algebra';
import { trajectoryFromMol } from '../../mol-model-formats/structure/mol';
import { Structure } from '../../mol-model/structure';
import { ArrayTrajectory } from '../../mol-model/structure/trajectory';
import { BallAndStickRepresentationProvider } from '../../mol-repr/structure/representation/ball-and-stick';
import { Task } from '../../mol-task';
import { ColorTheme } from '../../mol-theme/color';
import { SizeTheme } from '../../mol-theme/size';
export function getSdfTrajectory(url) {
    return __awaiter(this, void 0, void 0, function () {
        var data, parsed, _a, models, _i, _b, molFile, traj, i, _c, _d;
        return __generator(this, function (_e) {
            switch (_e.label) {
                case 0: return [4 /*yield*/, fetch(url)];
                case 1:
                    data = _e.sent();
                    _a = parseSdf;
                    return [4 /*yield*/, data.text()];
                case 2: return [4 /*yield*/, _a.apply(void 0, [_e.sent()]).run()];
                case 3:
                    parsed = _e.sent();
                    if (parsed.isError)
                        throw parsed;
                    models = [];
                    _i = 0, _b = parsed.result.compounds;
                    _e.label = 4;
                case 4:
                    if (!(_i < _b.length)) return [3 /*break*/, 10];
                    molFile = _b[_i].molFile;
                    return [4 /*yield*/, trajectoryFromMol(molFile).run()];
                case 5:
                    traj = _e.sent();
                    i = 0;
                    _e.label = 6;
                case 6:
                    if (!(i < traj.frameCount)) return [3 /*break*/, 9];
                    _d = (_c = models).push;
                    return [4 /*yield*/, Task.resolveInContext(traj.getFrameAtIndex(i))];
                case 7:
                    _d.apply(_c, [_e.sent()]);
                    _e.label = 8;
                case 8:
                    i++;
                    return [3 /*break*/, 6];
                case 9:
                    _i++;
                    return [3 /*break*/, 4];
                case 10: return [2 /*return*/, new ArrayTrajectory(models)];
            }
        });
    });
}
var colorThemeRegistry = ColorTheme.createRegistry();
var sizeThemeRegistry = SizeTheme.createRegistry();
export function getReference(traj, i) {
    return __awaiter(this, void 0, void 0, function () {
        var model, structure;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, Task.resolveInContext(traj.getFrameAtIndex(i))];
                case 1:
                    model = _a.sent();
                    structure = Structure.ofModel(model);
                    return [2 /*return*/, structure.boundary.sphere];
            }
        });
    });
}
export function initModel(i, canvas3d, traj, ref) {
    return __awaiter(this, void 0, void 0, function () {
        var reprCtx, model, _structure, translation, transform, structure, labelElem, ballAndStickRepr;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    reprCtx = { webgl: canvas3d.webgl, colorThemeRegistry: colorThemeRegistry, sizeThemeRegistry: sizeThemeRegistry };
                    return [4 /*yield*/, Task.resolveInContext(traj.getFrameAtIndex(i))];
                case 1:
                    model = _a.sent();
                    _structure = Structure.ofModel(model);
                    translation = Vec3.sub(Vec3(), ref, _structure.boundary.sphere.center);
                    transform = Mat4.fromTranslation(Mat4(), translation);
                    structure = Structure.transform(_structure, transform);
                    labelElem = document.getElementById("label" + i);
                    labelElem.innerText = model.entities.data.pdbx_description.toArray()[0].join(', ');
                    ballAndStickRepr = BallAndStickRepresentationProvider.factory(reprCtx, BallAndStickRepresentationProvider.getParams);
                    ballAndStickRepr.setTheme({
                        color: reprCtx.colorThemeRegistry.create('element-symbol', { structure: structure }, { carbonColor: 'elementSymbol' }),
                        size: reprCtx.sizeThemeRegistry.create('physical', { structure: structure }, { value: 1 })
                    });
                    return [4 /*yield*/, ballAndStickRepr.createOrUpdate(__assign(__assign({}, BallAndStickRepresentationProvider.defaultValues), { quality: 'medium' }), structure).run()];
                case 2:
                    _a.sent();
                    canvas3d.add(ballAndStickRepr);
                    return [2 /*return*/];
            }
        });
    });
}
//# sourceMappingURL=model.js.map