/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Segmentation } from '../../../mol-data/int/segmentation';
import { SortedRanges } from '../../../mol-data/int/sorted-ranges';
import { OrderedSet } from '../../../mol-data/int';
import { Vec3 } from '../../../mol-math/linear-algebra';
export function calcHelixOrientation(model) {
    console.log('calcHelixOrientation');
    var _a = model.atomicConformation, x = _a.x, y = _a.y, z = _a.z;
    var _b = model.atomicHierarchy.derived.residue, polymerType = _b.polymerType, traceElementIndex = _b.traceElementIndex;
    var n = polymerType.length;
    var elements = OrderedSet.ofBounds(0, model.atomicConformation.atomId.rowCount);
    var polymerIt = SortedRanges.transientSegments(model.atomicRanges.polymerRanges, elements);
    var residueIt = Segmentation.transientSegments(model.atomicHierarchy.residueAtomSegments, elements);
    var centers = new Float32Array(n * 3);
    var axes = new Float32Array(n * 3);
    var twists = new Float32Array(n);
    var diffs = new Float32Array(n);
    var rises = new Float32Array(n);
    var radii = new Float32Array(n);
    var i = 0;
    var j = -1;
    var s = -1;
    var a1 = Vec3();
    var a2 = Vec3();
    var a3 = Vec3();
    var a4 = Vec3();
    var r12 = Vec3();
    var r23 = Vec3();
    var r34 = Vec3();
    var v1 = Vec3();
    var v2 = Vec3();
    var vt = Vec3();
    var diff13 = Vec3();
    var diff24 = Vec3();
    var axis = Vec3();
    var prevAxis = Vec3();
    while (polymerIt.hasNext) {
        var ps = polymerIt.move();
        // console.log('polymer', ps);
        residueIt.setSegment(ps);
        i = -1;
        s = -1;
        while (residueIt.hasNext) {
            i += 1;
            var index = residueIt.move().index;
            if (i === 0)
                s = index;
            j = (index - 2);
            var j3 = j * 3;
            // console.log(index, i, j);
            Vec3.copy(a1, a2);
            Vec3.copy(a2, a3);
            Vec3.copy(a3, a4);
            var eI_1 = traceElementIndex[index];
            Vec3.set(a4, x[eI_1], y[eI_1], z[eI_1]);
            if (i < 3) {
                // Vec3.toArray(a1, centers, j3);
                // console.log('skip');
                continue;
            }
            Vec3.sub(r12, a2, a1);
            Vec3.sub(r23, a3, a2);
            Vec3.sub(r34, a4, a3);
            Vec3.sub(diff13, r12, r23);
            Vec3.sub(diff24, r23, r34);
            Vec3.cross(axis, diff13, diff24);
            Vec3.normalize(axis, axis);
            Vec3.toArray(axis, axes, j3);
            if (j > 0) {
                diffs[j] = Vec3.angle(axis, prevAxis);
            }
            var tmp = Math.cos(Vec3.angle(diff13, diff24));
            twists[j] = 180.0 / Math.PI * Math.acos(tmp);
            var diff13Length = Vec3.magnitude(diff13);
            var diff24Length = Vec3.magnitude(diff24);
            var r = (Math.sqrt(diff24Length * diff13Length) /
                // clamp, to avoid instabilities for when
                // angle between diff13 and diff24 is near 0
                Math.max(2.0, 2.0 * (1.0 - tmp)));
            radii[j] = r;
            rises[j] = Math.abs(Vec3.dot(r23, axis));
            Vec3.scale(v1, diff13, r / diff13Length);
            Vec3.sub(v1, a2, v1);
            Vec3.toArray(v1, centers, j3);
            Vec3.scale(v2, diff24, r / diff24Length);
            Vec3.sub(v2, a3, v2);
            Vec3.toArray(v2, centers, j3 + 3);
            Vec3.copy(prevAxis, axis);
            // console.log(j, {
            //     radius: radii[j],
            //     rise: rises[j]
            // });
        }
        var s3 = s * 3;
        // calc axis as dir of second and third center pos
        // project first traceAtom onto axis to get first center pos
        // v1.fromArray(center as any, 3)  // TODO
        // v2.fromArray(center as any, 6)  // TODO
        // _axis.subVectors(v1, v2).normalize()
        Vec3.fromArray(v1, centers, s3 + 3);
        Vec3.fromArray(v2, centers, s3 + 6);
        Vec3.normalize(axis, Vec3.sub(axis, v1, v2));
        // // _center.copy( res[ 0 ].getTraceAtom() );
        // a1.index = polymer.getAtomIndexByType(0, type)!  // TODO
        // _center.copy(a1 as any)  // TODO
        var sI = traceElementIndex[s];
        Vec3.set(a1, x[sI], y[sI], z[sI]);
        // vt.copy(a1 as any)  // TODO
        // projectPointOnVector(vt, _axis, v1)
        // vt.toArray(center as any, 0)  // TODO
        Vec3.copy(vt, a1);
        Vec3.projectPointOnVector(vt, vt, axis, v1);
        Vec3.toArray(vt, centers, s3);
        // Vec3.toArray(a1, centers, s3);
        var e = j + 2;
        var e3 = e * 3;
        // console.log(j, { s, e });
        // calc axis as dir of n-1 and n-2 center pos
        // project last traceAtom onto axis to get last center pos
        // v1.fromArray(center as any, 3 * n - 6)  // TODO
        // v2.fromArray(center as any, 3 * n - 9)  // TODO
        // _axis.subVectors(v1, v2).normalize()
        Vec3.fromArray(v1, centers, e3 - 3);
        Vec3.fromArray(v2, centers, e3 - 6);
        Vec3.normalize(axis, Vec3.sub(axis, v1, v2));
        // // _center.copy( res[ n - 1 ].getTraceAtom() );
        // a1.index = polymer.getAtomIndexByType(n - 1, type)!  // TODO
        // _center.copy(a1 as any)  // TODO
        var eI = traceElementIndex[e];
        Vec3.set(a1, x[eI], y[eI], z[eI]);
        // vt.copy(a1 as any)  // TODO
        // projectPointOnVector(vt, _axis, v1)
        // vt.toArray(center as any, 3 * n - 3)  // TODO
        Vec3.copy(vt, a1);
        Vec3.projectPointOnVector(vt, vt, axis, v1);
        Vec3.toArray(vt, centers, e3);
        // Vec3.toArray(a1, centers, e3);
        // Vec3.fromArray(a1, centers, s3 + 3);
        // Vec3.fromArray(a2, centers, s3 + 6);
        // Vec3.fromArray(a3, centers, s3 + 9);
        // const t = 2;
        // let prevHelix = false;
        // for (let k = s + 2, kl = e - 2; k < kl; ++k) {
        //     const k3 = k * 3;
        //     // Vec3.fromArray(a1, centers, k3 - 3);
        //     // Vec3.fromArray(a2, centers, k3);
        //     // Vec3.fromArray(a3, centers, k3 + 3);
        //     Vec3.copy(a1, a2);
        //     Vec3.copy(a2, a3);
        //     Vec3.copy(a3, a4);
        //     Vec3.fromArray(a4, centers, k3 + 6);
        //     const d12 = Vec3.distance(a1, a2);
        //     const d23 = Vec3.distance(a2, a3);
        //     // console.log(k, { d12, d23 }, radToDeg(diffs[k]));
        //     // if (d12 < t && d23 >= t) {
        //     // if (prevAxis ) {
        //     //     Vec3.sub(r12, a2, a1);
        //     //     // Vec3.add(a3, a3, r12);
        //     //     // Vec3.toArray(a3, centers, k3 + 3);
        //     //     Vec3.add(vt, a2, r12);
        //     //     Vec3.toArray(vt, centers, k3 + 3);
        //     //     Vec3.add(vt, vt, r12);
        //     //     Vec3.toArray(vt, centers, k3 + 6);
        //     //     console.log('MOIN');
        //     // }
        //     if (d12 < t || d23 < t) {
        //         prevHelix = true;
        //     } else {
        //         prevHelix = false;
        //     }
        //     // if (d12 < t && d23 < t) {
        //     //     console.log('h');
        //     //     Vec3.scale(a2, Vec3.add(a2, a1, Vec3.add(a2, a3, a2)), 1 / 3);
        //     //     Vec3.toArray(a2, centers, k3);
        //     // }
        // }
    }
    return {
        centers: centers,
        axes: axes,
        diffs: diffs
    };
}
//     private set(out: Vec3, a1: Vec3, a2: Vec3, a3: Vec3, a4: Vec3) {
//         Vec3.sub(this.r12, a2, a1);
//         Vec3.sub(this.r23, a3, a2);
//         Vec3.sub(this.r34, a4, a3);
//         Vec3.sub(this.diff13, this.r12, this.r23);
//         Vec3.sub(this.diff24, this.r23, this.r34);
//         // _axis.crossVectors(diff13, diff24).normalize()
//         // _axis.toArray(axis as any, j)  // TODO
//         // if (i > 0) {
//         //     diff[ i ] = _axis.angleTo(_prevAxis)
//         // }
//         const tmp = Math.cos(Vec3.angle(this.diff13, this.diff24));
//         // twist[ i ] = 180.0 / Math.PI * Math.acos(tmp)
//         const diff13Length = Vec3.magnitude(this.diff13);
//         const diff24Length = Vec3.magnitude(this.diff24);
//         const r = (
//             Math.sqrt(diff24Length * diff13Length) /
//             // clamp, to avoid instabilities for when
//             // angle between diff13 and diff24 is near 0
//             Math.max(2.0, 2.0 * (1.0 - tmp))
//         );
//         Vec3.scale(out, this.diff13, r / diff13Length);
//         Vec3.sub(out, a2, out);
//     }
//# sourceMappingURL=helix-orientation.js.map