/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Áron Samuel Kovács <aron.kovacs@mail.muni.cz>
 */
export var cutawayJfaStep_frag = "\nprecision highp float;\nprecision highp int;\nprecision highp sampler2D;\n\n// x: seed x coord, <0, 1>\n// y: seed y coord, <0, 1>\n// z: seed z coord, <0, 1>\n// w: self z coord, <0, 1>\n// if x (or y or z or w) is less than 0 then that pixel is 'empty'\nuniform sampler2D tCutaway;\nuniform vec2 uTexSize;\n\nuniform mat4 uInvProjection;\n\n// needed for stereo camera\n// x, y, width, height, all <0, 1>\nuniform vec4 uViewport;\n\nuniform vec2 uStep;\n\nuniform float uPMsz;\nuniform float uAngle;\nuniform float uEdgeRegionSize;\nuniform float uSlopeStartOffset;\n\nuniform vec2 uAspectRatio;\n\nuniform float uIsOrtho;\nuniform float uNear;\nuniform float uFar;\n\n#include common\n\nfloat slope(float tanAngle, float linearZ) {\n    return (uPMsz + linearZ) / tanAngle;\n}\n\nvec2 normalizedDistanceFromEdges(vec2 coords) {\n    return min(coords - uViewport.xy, uViewport.xy + uViewport.zw - coords) / uViewport.zw;\n}\n\nfloat angleEdgeCompression(vec2 coords) {\n    if (uEdgeRegionSize == 0.0) {\n        return uAngle;\n    }\n    vec2 terms = clamp(normalizedDistanceFromEdges(coords) / uEdgeRegionSize, 0.0, 1.0);\n    return uAngle * terms.x * terms.y;\n}\n\nfloat slopeOffsetEdgeCompression(float slopeOffset, vec2 coords) {\n    if (uEdgeRegionSize == 0.0) {\n        return slopeOffset;\n    }\n    vec2 terms = clamp(normalizedDistanceFromEdges(coords) / uEdgeRegionSize, 0.0, 1.0);\n    return slopeOffset * terms.x * terms.y;\n}\n\nfloat getPixelViewSize(vec3 coords, vec2 invTexSize) {\n    float viewX1 = screenSpaceToWorldSpace(coords - vec3(invTexSize.x * 0.5, 0.0, 0.0), uInvProjection).x;\n    float viewX2 = screenSpaceToWorldSpace(coords + vec3(invTexSize.x * 0.5, 0.0, 0.0), uInvProjection).x;\n    return abs(viewX2 - viewX1);\n}\n\nvoid main(void) {\n    vec2 invTexSize = 1.0 / uTexSize;\n    vec2 selfCoords = gl_FragCoord.xy * invTexSize;\n\n    float angle = max(angleEdgeCompression(selfCoords), 0.001);\n    float tanAngle = tan(angle);\n\n    vec4 result = texture(tCutaway, selfCoords);\n    float resultViewZ = depthToViewZ(uIsOrtho, result.w, uNear, uFar);\n    float resultLinearZ = (resultViewZ - uNear) / (uFar - uNear);\n    \n    for (int dy = -1; dy <= 1; dy++) {\n        for (int dx = -1; dx <= 1; dx++) {\n            if (dx == 0 && dy == 0) {\n                continue;\n            }\n\n            vec2 sampleCoords = selfCoords + vec2(float(dx), float(dy)) * uStep;\n            if (sampleCoords.x < uViewport.x ||\n                sampleCoords.x > uViewport.x + uViewport.z ||\n                sampleCoords.y < uViewport.y ||\n                sampleCoords.y > uViewport.y + uViewport.w) {\n                continue;\n            }\n\n            vec4 sampleValue = texture2D(tCutaway, sampleCoords);\n            if (sampleValue.x < 0.0) {\n                continue;                \n            }\n\n            float seedPixelViewSize = getPixelViewSize(sampleValue.xyz, invTexSize);\n            float slopeStartOffset = uSlopeStartOffset / (seedPixelViewSize * uTexSize.x);\n\n            vec2 coordDiff = (selfCoords.xy - sampleValue.xy) * uAspectRatio;\n            float dist = max(length(coordDiff) - slopeOffsetEdgeCompression(slopeStartOffset, selfCoords), 0.0);\n            \n            float seedViewZ = depthToViewZ(uIsOrtho, sampleValue.z, uNear, uFar);\n            float seedLinearZ = (seedViewZ - uNear) / (uFar - uNear);\n            float cutawayLinearZ = seedLinearZ - dist * slope(tanAngle, seedLinearZ);\n\n            if (result.x < 0.0 || (cutawayLinearZ <= 0.0 && resultLinearZ > cutawayLinearZ)) {\n                float cutawayViewZ = uNear + (cutawayLinearZ * (uFar - uNear));\n                float cutawayDepth = viewZToDepth(uIsOrtho, cutawayViewZ, uNear, uFar);\n\n                result = vec4(sampleValue.xyz, cutawayDepth);\n                resultLinearZ = cutawayLinearZ;\n            }\n        }\n    }\n\n    gl_FragColor = result;\n}\n";
//# sourceMappingURL=cutaway-jfa-step.frag.js.map